#This tests the self stopping timer, and turning it on.
#Initially, a state is turned on, but doesn't start the timer.
#The oneNeuron (with the state) turns the timer on.
#The timer should produce spikes throughout the cascade, that stop at the 
#last state. The state should fire persistently at 5ms.
import sys
import pyNN.nest as sim

from nealCoverClass import NealCoverFunctions
from stateMachineClass import FSAHelperFunctions
from timerClass import TimerClass

RUN_DURATION = 400
#initialize the simulator. 
def init():
    sim.setup(timestep=neal.DELAY,min_delay=neal.DELAY,
                    max_delay=neal.DELAY, debug=0)


def createStateCells():
    
    numNeurons = fsa.CA_SIZE
        
    cells=sim.Population(numNeurons,sim.IF_cond_exp,fsa.CELL_PARAMS)
    return cells

def createInputs():
    inputSpikeTimes0 = [10.0]
    spikeArray0 = {'spike_times': [inputSpikeTimes0]}
    spikeGen0=sim.Population(1,sim.SpikeSourceArray,spikeArray0,
                                   label='inputSpikes_0')
    inputSpikeTimes1 = [60.0]
    spikeArray1 = {'spike_times': [inputSpikeTimes1]}
    spikeGen1=sim.Population(1,sim.SpikeSourceArray,spikeArray1,
                                   label='inputSpikes_1')
    return [spikeGen0, spikeGen1]

def setupRecording(cells):
    cells.record({'spikes','v'})

def printResults(oneCell,stateCells):
    oneCell.write_data('results/tempOne.pkl')
    stateCells.write_data('results/tempState.pkl')

#---main 

simName = "nest"
spinnVersion = -1

neal = NealCoverFunctions(simName,sim,spinnVersion)
fsa = FSAHelperFunctions(simName,sim,neal,spinnVersion)

init()

#make neurons
oneCell=sim.Population(1,sim.IF_cond_exp,fsa.CELL_PARAMS)
setupRecording(oneCell)
stateCells = createStateCells()
setupRecording(stateCells)
timer = TimerClass(simName,sim,neal,spinnVersion,fsa,10)

spikeGenerators = createInputs()
spikeGenToState = spikeGenerators[0]
spikeGenToOneNeuron = spikeGenerators[1]

#connect things
timer.makeStopTimerSynapses()
fsa.makeCA(stateCells,0)

connector = [(0,0,fsa.INPUT_WEIGHT,neal.DELAY)]
neal.nealProjection(spikeGenToOneNeuron, oneCell, connector,'excitatory')
fsa.turnOnStateFromSpikeSource(spikeGenToState,stateCells,0)

timer.oneNeuronHalfStartsTimer(oneCell,0)
timer.stateHalfStartsTimer(stateCells,0)

neal.nealApplyProjections()
sim.run(RUN_DURATION)

timer.printResults("results/testTimer")
printResults(oneCell,stateCells)
